@@ -1,11 +1,12 @@
 import json
-import jsonschema
 import os
 
-from dice_ml.diverse_counterfactuals import CounterfactualExamples
-from dice_ml.utils.exception import UserConfigValidationException
-from dice_ml.diverse_counterfactuals import _DiverseCFV2SchemaConstants
+import jsonschema
+
 from dice_ml.constants import _SchemaVersions
+from dice_ml.diverse_counterfactuals import (CounterfactualExamples,
+                                             _DiverseCFV2SchemaConstants)
+from dice_ml.utils.exception import UserConfigValidationException
 
 
 class _CommonSchemaConstants:
@@ -165,7 +166,7 @@ class CounterfactualExplanations:
                 combined_test_instance_list.append(serialized_cf_examples[
                     _DiverseCFV2SchemaConstants.TEST_INSTANCE_LIST])
                 combined_final_cfs_list.append(serialized_cf_examples[
-                    _DiverseCFV2SchemaConstants.FIANL_CFS_LIST])
+                    _DiverseCFV2SchemaConstants.FINAL_CFS_LIST])
                 data_interface = serialized_cf_examples[
                     _DiverseCFV2SchemaConstants.DATA_INTERFACE]
                 feature_names = serialized_cf_examples[
@@ -251,7 +252,7 @@ class CounterfactualExplanations:
                     # counterfactual example.
                     cf_examples_str = json.dumps(
                         {
-                            _DiverseCFV2SchemaConstants.FIANL_CFS_LIST: json_dict[
+                            _DiverseCFV2SchemaConstants.FINAL_CFS_LIST: json_dict[
                                 _CounterfactualExpV2SchemaConstants.CFS_LIST][index],
                             _DiverseCFV2SchemaConstants.TEST_INSTANCE_LIST: json_dict[
                                 _CounterfactualExpV2SchemaConstants.TEST_DATA][index],
