@@ -156,7 +156,6 @@ class ExplainerBase(ABC):
                 query_instances_list.append(query_instances[ix:(ix+1)])
         elif isinstance(query_instances, Iterable):
             query_instances_list = query_instances
-
         for query_instance in tqdm(query_instances_list):
             self.data_interface.set_continuous_feature_indexes(query_instance)
             res = self._generate_counterfactuals(
@@ -171,7 +170,6 @@ class ExplainerBase(ABC):
                 verbose=verbose,
                 **kwargs)
             cf_examples_arr.append(res)
-
         self._check_any_counterfactuals_computed(cf_examples_arr=cf_examples_arr)
 
         return CounterfactualExplanations(cf_examples_list=cf_examples_arr)
@@ -227,9 +225,6 @@ class ExplainerBase(ABC):
 
         self.check_query_instance_validity(features_to_vary, permitted_range, query_instance, feature_ranges_orig)
 
-        # check feature MAD validity and throw warnings
-        self.data_interface.check_mad_validity(feature_weights)
-
         return features_to_vary
 
     def check_query_instance_validity(self, features_to_vary, permitted_range, query_instance, feature_ranges_orig):
@@ -239,7 +234,6 @@ class ExplainerBase(ABC):
 
             if feature not in self.data_interface.feature_names:
                 raise ValueError("Feature", feature, "not present in training data!")
-
         for feature in self.data_interface.categorical_feature_names:
             if query_instance[feature].values[0] not in feature_ranges_orig[feature] and \
                     str(query_instance[feature].values[0]) not in feature_ranges_orig[feature]:
@@ -486,14 +480,19 @@ class ExplainerBase(ABC):
 
     def predict_fn(self, input_instance):
         """prediction function"""
-        return self.model.get_output(input_instance)
+
+        preds = self.model.get_output(input_instance)
+        if self.model.model_type == ModelTypes.Classifier and \
+           len(preds.shape) == 1:  # from deep learning predictors
+            preds = np.column_stack([1 - preds, preds])
+        return preds
 
     def predict_fn_for_sparsity(self, input_instance):
         """prediction function for sparsity correction"""
         return self.model.get_output(input_instance)
 
     def do_posthoc_sparsity_enhancement(self, final_cfs_sparse, query_instance, posthoc_sparsity_param,
-                                        posthoc_sparsity_algorithm):
+                                        posthoc_sparsity_algorithm, limit_steps_ls):
         """Post-hoc method to encourage sparsity in a generated counterfactuals.
 
         :param final_cfs_sparse: Final CFs in original user-fed format, in a pandas dataframe.
@@ -504,6 +503,8 @@ class ExplainerBase(ABC):
                                            large (for instance, income varying from 10k to 1000k)
                                            and only if the features share a monotonic relationship
                                            with predicted outcome in the model.
+        :param limit_steps_ls: Defines the limit of steps to be done in the linear search,
+                                necessary to avoid infinite loops
         """
         if final_cfs_sparse is None:
             return final_cfs_sparse
@@ -535,28 +536,31 @@ class ExplainerBase(ABC):
                 if(abs(diff) <= quantiles[feature]):
                     if posthoc_sparsity_algorithm == "linear":
                         final_cfs_sparse = self.do_linear_search(diff, decimal_prec, query_instance, cf_ix,
-                                                                 feature, final_cfs_sparse, current_pred)
+                                                                 feature, final_cfs_sparse, current_pred, limit_steps_ls)
 
                     elif posthoc_sparsity_algorithm == "binary":
                         final_cfs_sparse = self.do_binary_search(
                             diff, decimal_prec, query_instance, cf_ix, feature, final_cfs_sparse, current_pred)
 
             temp_preds = self.predict_fn_for_sparsity(final_cfs_sparse.loc[[cf_ix]][self.data_interface.feature_names])
-            cfs_preds_sparse.append(temp_preds)
-
+            cfs_preds_sparse.append(temp_preds[0])
         final_cfs_sparse[self.data_interface.outcome_name] = self.get_model_output_from_scores(cfs_preds_sparse)
         # final_cfs_sparse[self.data_interface.outcome_name] = np.round(final_cfs_sparse[self.data_interface.outcome_name], 3)
         return final_cfs_sparse
 
-    def do_linear_search(self, diff, decimal_prec, query_instance, cf_ix, feature, final_cfs_sparse, current_pred_orig):
+    def do_linear_search(self, diff, decimal_prec, query_instance, cf_ix, feature, final_cfs_sparse,
+                         current_pred_orig, limit_steps_ls):
         """Performs a greedy linear search - moves the continuous features in CFs towards original values in
-           query_instance greedily until the prediction class changes."""
+           query_instance greedily until the prediction class changes, or it reaches the maximum number of steps"""
 
         old_diff = diff
         change = (10**-decimal_prec[feature])  # the minimal possible change for a feature
         current_pred = current_pred_orig
+        count_steps = 0
         if self.model.model_type == ModelTypes.Classifier:
-            while((abs(diff) > 10e-4) and (np.sign(diff*old_diff) > 0) and self.is_cf_valid(current_pred)):
+            while((abs(diff) > 10e-4) and (np.sign(diff*old_diff) > 0) and
+                  self.is_cf_valid(current_pred)) and (count_steps < limit_steps_ls):
+
                 old_val = int(final_cfs_sparse.at[cf_ix, feature])
                 final_cfs_sparse.at[cf_ix, feature] += np.sign(diff)*change
                 current_pred = self.predict_fn_for_sparsity(final_cfs_sparse.loc[[cf_ix]][self.data_interface.feature_names])
@@ -569,6 +573,8 @@ class ExplainerBase(ABC):
 
                 diff = query_instance[feature].iat[0] - int(final_cfs_sparse.at[cf_ix, feature])
 
+                count_steps += 1
+
         return final_cfs_sparse
 
     def do_binary_search(self, diff, decimal_prec, query_instance, cf_ix, feature, final_cfs_sparse, current_pred):
@@ -651,14 +657,30 @@ class ExplainerBase(ABC):
         """
         if desired_class_input == "opposite":
             if num_output_nodes == 2:
-                original_pred_1 = np.argmax(original_pred)
+                # 'original_pred' needs to be converted to the proper class if 'original_pred' comes from the
+                # 'predict_proba' method (length is 2 and class is index with maximum value). Otherwise 'original_pred'
+                # already contains the class.
+                if hasattr(original_pred, "__len__") and len(original_pred) > 1:
+                    original_pred_1 = np.argmax(original_pred)
+                else:
+                    original_pred_1 = original_pred
                 target_class = int(1 - original_pred_1)
                 return target_class
+            elif num_output_nodes == 1:  # only for pytorch DL model
+                original_pred_1 = np.round(original_pred)
+                target_class = int(1-original_pred_1)
+                return target_class
             elif num_output_nodes > 2:
                 raise UserConfigValidationException(
                     "Desired class cannot be opposite if the number of classes is more than 2.")
         elif isinstance(desired_class_input, int):
-            if desired_class_input >= 0 and desired_class_input < num_output_nodes:
+            if num_output_nodes == 1:   # for DL models
+                if desired_class_input in (0, 1):
+                    target_class = desired_class_input
+                    return target_class
+                else:
+                    raise UserConfigValidationException("Only 0, 1 are supported as desired class for binary classification!")
+            elif desired_class_input >= 0 and desired_class_input < num_output_nodes:
                 target_class = desired_class_input
                 return target_class
             else:
@@ -683,11 +705,15 @@ class ExplainerBase(ABC):
         for i in range(len(model_outputs)):
             pred = model_outputs[i]
             if self.model.model_type == ModelTypes.Classifier:
-                if self.num_output_nodes == 2:  # binary
-                    pred_1 = pred[self.num_output_nodes-1]
+                if self.num_output_nodes in (1, 2):  # binary
+                    if self.num_output_nodes == 2:
+                        pred_1 = pred[self.num_output_nodes-1]
+                    else:
+                        pred_1 = pred[0]
                     validity[i] = 1 if \
                         ((self.target_cf_class == 0 and pred_1 <= self.stopping_threshold) or
                          (self.target_cf_class == 1 and pred_1 >= self.stopping_threshold)) else 0
+
                 else:  # multiclass
                     if np.argmax(pred) == self.target_cf_class:
                         validity[i] = 1
@@ -714,14 +740,14 @@ class ExplainerBase(ABC):
                     target_cf_class = self.target_cf_class[0][0]
             target_cf_class = int(target_cf_class)
 
-            if self.num_output_nodes == 1:  # for tensorflow/pytorch models
+            if len(model_score) == 1:  # for tensorflow/pytorch models
                 pred_1 = model_score[0]
                 validity = True if \
                     ((target_cf_class == 0 and pred_1 <= self.stopping_threshold) or
                      (target_cf_class == 1 and pred_1 >= self.stopping_threshold)) else False
                 return validity
-            if self.num_output_nodes == 2:  # binary
-                pred_1 = model_score[self.num_output_nodes-1]
+            elif len(model_score) == 2:  # binary
+                pred_1 = model_score[1]
                 validity = True if \
                     ((target_cf_class == 0 and pred_1 <= self.stopping_threshold) or
                      (target_cf_class == 1 and pred_1 >= self.stopping_threshold)) else False
@@ -739,7 +765,13 @@ class ExplainerBase(ABC):
         model_output = np.zeros(len(model_scores), dtype=output_type)
         for i in range(len(model_scores)):
             if self.model.model_type == ModelTypes.Classifier:
-                model_output[i] = np.argmax(model_scores[i])
+                if hasattr(model_scores[i], "shape") and len(model_scores[i].shape) > 0:
+                    if model_scores[i].shape[0] > 1:
+                        model_output[i] = np.argmax(model_scores[i])
+                    else:
+                        model_output[i] = np.round(model_scores[i])[0]
+                else:  # 1-D input
+                    model_output[i] = np.round(model_scores[i])
             elif self.model.model_type == ModelTypes.Regressor:
                 model_output[i] = model_scores[i]
         return model_output
@@ -770,7 +802,7 @@ class ExplainerBase(ABC):
         dataset_instance = self.data_interface.prepare_query_instance(
             query_instance=data_df_copy[self.data_interface.feature_names])
 
-        predictions = self.model.model.predict(dataset_instance)
+        predictions = self.model.get_output(dataset_instance, model_score=False).flatten()
         # TODO: Is it okay to insert a column in the original dataframe with the predicted outcome? This is memory-efficient
         data_df_copy[predicted_outcome_name] = predictions
 
