@@ -112,9 +112,9 @@ class TestCounterfactualExplanations:
 @pytest.fixture()
 def random_binary_classification_exp_object():
     backend = 'sklearn'
-    dataset = helpers.load_custom_testing_dataset()
+    dataset = helpers.load_custom_testing_dataset_binary()
     d = dice_ml.Data(dataframe=dataset, continuous_features=['Numerical'], outcome_name='Outcome')
-    ML_modelpath = helpers.get_custom_dataset_modelpath_pipeline()
+    ML_modelpath = helpers.get_custom_dataset_modelpath_pipeline_binary()
     m = dice_ml.Model(model_path=ML_modelpath, backend=backend)
     exp = dice_ml.Dice(d, m, method='random')
     return exp
